
/*
** This is an example to show you how to use the MixerKernel.c functions.
** You just have to 'make MixerKernelTest' to compile it.
*/

#include<stdio.h>
#include<stdlib.h>
#include<unistd.h>
#include"MixerTypes.h"
#include"MixerKernel.h"
#include"ConfigLib.h"
#include"DspLib.h"

void main(int argc, char **argv[])

{
  t_dsp*        my_dsp;
  t_main_mixer* main_mixer;    /* The main_mixer */
  t_mixer_elt*  my_first_elt;  /* A mixer element */
  t_mixer_elt*  my_second_elt; /* Another one */
  t_tempgmurf*  my_tempgmurf;  /* A temp file */
  t_mixer_info* my_mixerinfo;  /* Wave informations */
  t_gmurfrc*    my_gmurfrc;    /* Config File */

  t_point*      the_first_point;
  t_point*      the_second_point;

  int j;

  /* Initializing the Config Structure with defaults */
  my_gmurfrc = gmurfrc_defaults();

  /* Initializing DSP with the Config Structure */
  my_dsp = open_dsp_for_play(my_gmurfrc->first_dsp->name, 
		    my_gmurfrc->first_dsp->bits_per_sample,
		    my_gmurfrc->first_dsp->channels,
		    my_gmurfrc->first_dsp->sample_rate);

  if (my_dsp == NULL) /* The DSP couldn't be initialized. */
    {
      printf("Error while initializing %s\n",my_gmurfrc->first_dsp->name);
      exit(-1);
    }

  /* Constructing the Main Mixer structure. */
  main_mixer = new_main_mixer(my_dsp,5,50,44100,32);

  /* Choosing the temp file for the first element. */
  my_tempgmurf = new_tempgmurf("../MiniProg/tarzan.tempgmurf");

  /* Choosing its properties. */
  if (argc <= 1) {
    my_mixerinfo = new_mixer_info(255,44100,5000,0);
  } else {
    printf("Setting beginning date for %s to %s\n",my_tempgmurf->tempname,argv[1]);
    my_mixerinfo = new_mixer_info(255,44100,5000,atoi(argv[1]));
  }

  /* Creating the first mixer element. */
  my_first_elt = add_mixer_elt(main_mixer,  my_dsp, my_tempgmurf, 
			       my_mixerinfo);

  /* Creating another mixer element... */
  my_tempgmurf = new_tempgmurf("../MiniProg/Microsoft.tempgmurf");
  my_mixerinfo = new_mixer_info(255,44100,50,0);
  my_second_elt = add_mixer_elt(main_mixer, my_dsp, my_tempgmurf, 
				my_mixerinfo);

  /* Adding volume curse... */
  printf("Adding curses...\n");
  the_first_point  = new_vol_curse(my_second_elt, 0, 0);
  the_second_point = add_point(my_second_elt->mixer_info->vol_curse, 100000, 256);
  add_point(the_second_point, 150000, 0);

  /*
  ** Processing
  */
  
  printf("Initializing mixer...\n");
  /* Resetting playing position. */
  main_mixer->played_samples = 0;

  /* Mixer initialization for playing. */
  mixer_initialize(main_mixer);
  printf("Done.\n");

  /* Processing sound */
  printf("Processing sound...\n");
  while ((j = mixer_update_buffer(main_mixer)) != 0)
    {
      /* Nothing to do. */
    }
  printf("Done.\n");

  /* Unlocks the DSPs that are used by the mixer. */
  mixer_release(main_mixer);

  /*
  ** Freeing structures from memory
  */

  /* Freeing main mixer. */
  main_mixer_destroy(main_mixer);

  /* Freeing Config Structure. */
  gmurfrc_destroy(my_gmurfrc);

  /* Closing DSP File. */
  close_dsp(my_dsp);

}
